'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var fs = require('fs');
var jspythonInterpreter = require('jspython-interpreter');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var fs__default = /*#__PURE__*/_interopDefaultLegacy(fs);

/*! *****************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */

function __awaiter(thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
}

function __classPrivateFieldGet(receiver, state, kind, f) {
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
    return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
}

function __classPrivateFieldSet(receiver, state, value, kind, f) {
    if (kind === "m") throw new TypeError("Private method is not writable");
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a setter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot write private member to an object whose class did not declare it");
    return (kind === "a" ? f.call(receiver, value) : f ? f.value = value : state.set(receiver, value)), value;
}

var _Assert_chainedCheckCount, _Assert_logFn;
class Assert {
    constructor(assert, dataContext, logFn) {
        this.assert = assert;
        this.dataContext = dataContext;
        _Assert_chainedCheckCount.set(this, 1);
        this.status = true;
        _Assert_logFn.set(this, void 0);
        __classPrivateFieldSet(this, _Assert_logFn, logFn, "f");
    }
    equal(expected, received) {
        return this.assertFunction(expected, received, (e, r) => {
            if (typeof e === 'object') {
                // deepClone
                return JSON.stringify(e) !== JSON.stringify(r);
            }
            return e === r;
        }, (e, r) => `Expected '${e}', received '${r}'`);
    }
    notEqual(expected, received) {
        return this.assertFunction(expected, received, (e, r) => {
            if (typeof e === 'object') {
                // deepClone
                return JSON.stringify(e) !== JSON.stringify(r);
            }
            return e !== r;
        }, (e, r) => `Expected '${e}' is the same as received '${r}'`);
    }
    isTruthy(value) {
        return this.assertFunction(value, null, (e, r) => !!e, (e, r) => `Value '${e}' is not Truthy - https://developer.mozilla.org/en-US/docs/Glossary/Truthy`);
    }
    isFalsy(value) {
        return this.assertFunction(value, null, (e, r) => !e, (e, r) => `Value '${e}' is not Falsy - https://developer.mozilla.org/en-US/docs/Glossary/Falsy`);
    }
    isTrue(value) {
        return this.assertFunction(value, null, (e, r) => e === true, (e, r) => `Value '${e}' is not true`);
    }
    isFalse(value) {
        return this.assertFunction(value, null, (e, r) => e === false, (e, r) => `Value '${e}' is not false`);
    }
    greaterThan(value1, value2) {
        return this.assertFunction(value1, value2, (e, r) => e > r, (e, r) => `${e}' is not greater than ${r}`);
    }
    greaterOrEqualThan(value1, value2) {
        return this.assertFunction(value1, value2, (e, r) => e >= r, (e, r) => `${e}' is not greater (or equal) than ${r}`);
    }
    inRange(value, min, max) {
        return this.between(value, min, max);
    }
    between(value, min, max) {
        return this.assertFunction(value, { min, max }, (v, r) => v >= r.min && v <= r.max, (v, r) => `${v}' is NOT in range of ${r.min} and ${r.max}`);
    }
    lessThan(value1, value2) {
        return this.assertFunction(value1, value2, (e, r) => e < r, (e, r) => `${e}' is not lesser than ${r}`);
    }
    lessOrEqualThan(value1, value2) {
        return this.assertFunction(value1, value2, (e, r) => e <= r, (e, r) => `${e}' is not lesser (or equal) than ${r}`);
    }
    assertFunction(expected, received, assertExpressionFn, errorMessageFn) {
        var _a;
        // resolve function, if any
        if (typeof expected === 'function') {
            expected = expected(this.dataContext);
        }
        if (typeof received === 'function') {
            received = received(this.dataContext);
        }
        if (!assertExpressionFn(expected, received)) {
            this.status = false;
            this.message = `[${__classPrivateFieldGet(this, _Assert_chainedCheckCount, "f")}] - ${errorMessageFn(expected, received)}`;
            __classPrivateFieldGet(this, _Assert_logFn, "f").call(this, false, this.message);
            return this;
        }
        __classPrivateFieldGet(this, _Assert_logFn, "f").call(this, true, '');
        __classPrivateFieldSet(this, _Assert_chainedCheckCount, (_a = __classPrivateFieldGet(this, _Assert_chainedCheckCount, "f"), _a++, _a), "f");
        return this;
    }
}
_Assert_chainedCheckCount = new WeakMap(), _Assert_logFn = new WeakMap();

function trimChar(text, charToRemove) {
    while (text.charAt(0) == charToRemove) {
        text = text.substring(1);
    }
    while (text.charAt(text.length - 1) == charToRemove) {
        text = text.substring(0, text.length - 1);
    }
    return text;
}

const rootFolder = process.cwd().split('\\').join('/');
const initialScope = {
    __name__: 'main',
    session: {},
    asserts: []
};
let previousLogMessage = '';
const context = {
    params: {}
};
function logFn(msg) {
    const level = msg.level === 'success' ? msg.level : msg.level + '   ';
    const message = `${level} | ${msg.message}`;
    if (message !== previousLogMessage) {
        console.log(`| ${msg.time.toTimeString().slice(0, 8)} | ${message}`);
        previousLogMessage = message;
    }
}
function assert(name, dataContext) {
    // an original case when
    if (typeof dataContext === 'boolean') {
        logFn({
            level: dataContext ? 'success' : 'fail',
            message: name || '',
            time: new Date()
        });
        initialScope.asserts.push({ success: !!dataContext, name });
        return;
    }
    function assertCallback(success, message) {
        var _a;
        logFn({
            logId: name,
            level: success ? 'success' : 'fail',
            message: `${name} ${message ? ':' : ''} ${message || ''}`,
            time: new Date()
        });
        const existingAssert = (_a = initialScope.asserts) === null || _a === void 0 ? void 0 : _a.find((a) => a.name === name);
        if (existingAssert) {
            // only if condition it is not fail yet
            if (!!existingAssert.success) {
                existingAssert.success = !!success;
                existingAssert.description = message;
            }
        }
        else {
            initialScope.asserts.push({ success: !!success, name: name, description: message });
        }
    }
    return new Assert(name, dataContext, assertCallback);
}
function getScript(fileName) {
    if (!fs__default["default"].existsSync(fileName)) {
        throw Error(`File not found`);
    }
    const scripts = fs__default["default"].readFileSync(fileName, 'utf8');
    return scripts;
}
function initialize(baseSource) {
    return __awaiter(this, void 0, void 0, function* () {
        // process app.js (if exists)
        //  - run _init
        //  - delete _ init
        //  - run _initAsync
        //  - delete _initAsync
        //  - load content into 'app'
        let appJsPath = `${rootFolder}/${baseSource}app.js`;
        console.log({ rootFolder, baseSource });
        if (!fs__default["default"].existsSync(appJsPath)) {
            appJsPath = `${rootFolder}/src/app.js`;
        }
        if (fs__default["default"].existsSync(appJsPath)) {
            const app = require(appJsPath);
            if (typeof app._init == 'function') {
                app._init();
                delete app._init;
            }
            if (typeof app._initAsync == 'function') {
                yield app._initAsync();
                delete app._initAsync;
            }
            Object.assign(initialScope, app);
        }
    });
}
initialScope.assert = (name, dataContext) => assert(name, dataContext);
initialScope.showAsserts = () => {
    var _a;
    return console.table((_a = initialScope.asserts) === null || _a === void 0 ? void 0 : _a.map((r) => ({
        status: r.success ? 'success' : 'fail',
        assert: `${r.name}${!!r.description ? ': ' : ''}${r.description || ''}`.trim()
    })));
};
initialScope.print = (...args) => logFn({
    time: new Date(),
    level: 'info',
    message: args.map((v) => (typeof v === 'object' ? JSON.stringify(v) : v)).join(' ')
});
function jsPythonForNode(options = {
    srcRoot: ''
}) {
    return __awaiter(this, void 0, void 0, function* () {
        const interpreter = jspythonInterpreter.jsPython();
        Object.assign(context.params, options.params);
        yield initialize(options.srcRoot || '');
        interpreter
            .registerPackagesLoader(packageLoader)
            .registerModuleLoader(moduleLoader);
        const evaluate = interpreter.evaluate;
        interpreter.evaluate = function (script, evaluationContext, entryFunctionName, moduleName) {
            return __awaiter(this, void 0, void 0, function* () {
                initialScope.asserts.splice(0, initialScope.asserts.length);
                return evaluate.call(interpreter, script, evaluationContext, entryFunctionName, moduleName);
            });
        };
        interpreter.evaluateFile = function (filePath, context = {}) {
            const script = getScript(filePath);
            return interpreter.evaluate(script, context, options.entryFunction);
        };
        return interpreter;
        function moduleLoader(filePath) {
            filePath = trimChar(trimChar(filePath, '/'), '.');
            let fileName = `${options.srcRoot || ''}${filePath}.jspy`;
            if (!fs__default["default"].existsSync(fileName)) {
                fileName = `${options.srcRoot || ''}${filePath}`;
            }
            if (!fs__default["default"].existsSync(fileName)) {
                fileName = `src/${filePath}`;
            }
            try {
                const script = fs__default["default"].readFileSync(fileName, 'utf8');
                return Promise.resolve(script);
            }
            catch (e) {
                console.log('* module loader error ', (e === null || e === void 0 ? void 0 : e.message) || e);
                return Promise.reject(e);
            }
        }
        /**@type {PackageLoader} */
        function packageLoader(packageName) {
            var _a;
            try {
                if ([
                    'fs',
                    'path',
                    'readline',
                    'timers',
                    'child_process',
                    'util',
                    'zlib',
                    'stream',
                    'net',
                    'https',
                    'http',
                    'events',
                    'os',
                    'buffer'
                ].includes(packageName)) {
                    return require(packageName);
                }
                if (packageName.toLowerCase().endsWith('.js') ||
                    packageName.toLowerCase().endsWith('.json')) {
                    return require(`${rootFolder}/${options.srcRoot || ''}${packageName}`);
                }
                return require(`${rootFolder}/node_modules/${packageName}`);
            }
            catch (err) {
                console.log('Import Error: ', (_a = err === null || err === void 0 ? void 0 : err.message) !== null && _a !== void 0 ? _a : err);
                throw err;
            }
        }
    });
}

exports.initialScope = initialScope;
exports.jsPythonForNode = jsPythonForNode;
